/* ----  Copyright (C) 1997 Alex Solomonoff ( asolomon@bbn.com )
This file is part of Invariant, a library of C++ tools for debugging and 
program consistancy checking. Invariant is free software; you can redistribute
it and/or modify it under the terms of the GNU Library General Public License,
version 2, as published by the Free Software Foundation.  ----- */

#ifndef _ASSERT_H_
#define _ASSERT_H_

#include <iostream.h>  // I wish I could put include guards on this.

#define NULL_EXPR ((void)0)

enum invl_assert_modes { assert_die, assert_msg, assert_quiet };

#ifdef NDEBUG
#	undef invl_assert
#	define invl_assert(EX) NULL_EXPR
#else
# 	ifdef NO_ASSERT_EXPRSSN_MSG
#		define invl_assert(EX)  ((EX) ? NULL_EXPR :   \
			_invl_nomsg_assert( __FILE__, __LINE__))
#	else
#	define invl_assert(EX) ((EX)? NULL_EXPR : _invl_assert( #EX, \
					__FILE__, (int)__LINE__))
#	endif
#endif

#ifdef USE_INVARIANT_ASSERT
#	define assert(cond) invl_assert(cond)
#endif

#ifndef NDEBUG
#       define DEBUG_CODE(expr)    expr
#else
#       define DEBUG_CODE(expr)    NULL_EXPR
#endif

#ifndef NDEBUG
#       define NOT_REACHED  _invl_assert("reached unreachable code",  \
                                 __FILE__, __LINE__)
#else
#       define NOT_REACHED  NULL_EXPR
#endif

invl_assert_modes set_assert_mode(invl_assert_modes mode);
ostream *	set_assert_stream( ostream * pos);
void		_invl_assert( const char * expression,
			 const char * filename, int line_number);
void		_invl_nomsg_assert( const char * filename, int line_number);
bool		realnum_is_reasonable(double d);
bool		realnum_is_reasonable(float d);

void		sigh();

const char *	invl_version_string();

#ifndef _ARITH_SIGNAL_H_
#	include <invariant/arith_signal.h>
#endif

/*------------ Sigh, cannot get these to work ------------
void		set_double_array_nan(double * arr, int n);
void		set_float_array_nan(float * arr, int n);
 *-------------------------------------------------------*/

#endif //!_ASSERT_H_
