/* ----  Copyright (C) 1997 Alex Solomonoff ( asolomon@bbn.com )
This file is part of Invariant, a library of C++ tools for debugging and 
program consistancy checking. Invariant is free software; you can redistribute
it and/or modify it under the terms of the GNU Library General Public License,
version 2, as published by the Free Software Foundation.  ----- */

#ifndef _ASSERT_USUSALLY_H_
#define _ASSERT_USUSALLY_H_

#ifndef _ASSERT_H_
#include <invariant/assert.h>
#endif

class leaky_bucket_counter
{
	const int	max_failures;
	const int	fail_period;
	int		failure_counter;
	int		period_counter;
    public:
	leaky_bucket_counter(int max_fail, int period)
	:	max_failures(max_fail),
		fail_period(period),
		failure_counter(max_fail),
		period_counter(period)
	{}
	bool operator()(bool expr)
	{	if ( !expr) failure_counter--;
		if (failure_counter<=0)
			return false;
		period_counter--;
		if ( period_counter==0)
		{	period_counter=fail_period;
			failure_counter=max_failures;
		}
		return true;
	}
};

#ifdef NDEBUG
#	define  assert_usually(max, period, expression)	NULL_EXPR
#else
#	ifdef NO_ASSERT_EXPRSSN_MSG
#		define assert_usually(max, period, expression)		\
		do {	static leaky_bucket_counter lbc(max, period);   \
			if ( !lbc(expression) )				\
			     _invl_nomsg_assert(__FILE__, (int)__LINE__);	\
		   } while(false)
#	else
#		define assert_usually(max, period, expression)		\
		do {	static leaky_bucket_counter lbc(max, period);	\
			if ( !lbc(expression) )				\
				_invl_assert(#expression,			\
					 __FILE__, (int)__LINE__);	\
		} while(false)
#	endif
#endif

#endif //! _ASSERT_USUSALLY_H_
