/* ----  Copyright (C) 1997 Alex Solomonoff ( asolomon@bbn.com )
This file is part of Invariant, a library of C++ tools for debugging and 
program consistancy checking. Invariant is free software; you can redistribute
it and/or modify it under the terms of the GNU Library General Public License,
version 2, as published by the Free Software Foundation.  ----- */

#ifndef _SAFE_CAST_H_
#define _SAFE_CAST_H_

// sigh, cannot put double include guard on system header.
#include <limits.h>

#ifndef _ASSERT_H_
#	include <invariant/assert.h>
#endif

// should these functions take const reference arguments?

template < class Num >
inline char
char_cast( Num n )
{
	invl_assert( n <= CHAR_MAX);
	invl_assert( n >= CHAR_MIN);
	return (char) n;
}

template < class Num >
inline unsigned char
unsigned_char_cast( Num n )
{
	invl_assert( n <= UCHAR_MAX);
	invl_assert( n >= UCHAR_MIN);
	return (unsigned char) n;
}

template < class Num >
inline signed char
signed_char_cast( Num n )
{
	invl_assert( n <= SCHAR_MAX);
	invl_assert( n >= SCHAR_MIN);
	return (signed char) n;
}

template < class Num >
inline short
short_cast( Num n )
{
	invl_assert( n <= SHRT_MAX);
	invl_assert( n >= SHRT_MIN);
	return (short) n;
}

template < class Num >
inline unsigned short
unsigned_short_cast( Num n )
{
	invl_assert( n <= USHRT_MAX);
	invl_assert( n >= 0);
	return (unsigned short) n;
}

template < class Num >
inline int 
int_cast( Num n )
{
	invl_assert( n <= INT_MAX);
	invl_assert( n >= INT_MIN);
	return (int) n;
}

template < class Num >
inline unsigned int 
unsigned_int_cast( Num n )
{
	invl_assert( n <= UINT_MAX);
	invl_assert( n >= 0       );
	return (unsigned int) n;
}

template < class Num >
inline long
long_cast( Num n )
{
	invl_assert( n <= LONG_MAX);
	invl_assert( n >= LONG_MIN);
	return (long) n;
}

template < class Num >
inline unsigned long
unsigned_long_cast( Num n )
{
	invl_assert( n <= ULONG_MAX);
	invl_assert( n >= ULONG_MIN);
	return (unsigned long) n;
}

#endif //! _SAFE_CAST_H_
