/* ----  Copyright (C) 1997 Alex Solomonoff ( asolomon@bbn.com )
This file is part of Invariant, a library of C++ tools for debugging and 
program consistancy checking. Invariant is free software; you can redistribute
it and/or modify it under the terms of the GNU Library General Public License,
version 2, as published by the Free Software Foundation.  ----- */

#include <iostream.h>
#include "invariant/assert.h"
#include <cmath>

typedef invl_assert_modes amodes;

static amodes assert_mode = assert_die;
static ostream * assert_stream = &cerr;

amodes
set_assert_mode( amodes  mode)
{
	amodes prev_mode = assert_mode;
	assert_mode = mode;
	return prev_mode;
}

ostream *
set_assert_stream( ostream * pos)
{	ostream *  prev_stream = assert_stream;
	assert_stream = pos;
	return prev_stream;
}

void
_invl_assert( const char * expression, const char * filename, int line_number)
{
	if ( assert_mode != assert_quiet)
		(*assert_stream) << "assertion failure: " << expression << " -- "
			     << filename << ", line " << line_number << endl;

	if (assert_mode==assert_die)
	{	(*assert_stream) << "exiting" << endl;
		exit(1);
	}
}

void
_invl_nomsg_assert( const char * filename, int line_number)
{	_invl_assert( "", filename, line_number);
}



bool
realnum_is_reasonable(double d)
{
	if ( isnan(d)  || !finite(d) )
		return false;

 	double fd = fabs(d);
	if ( (fd > 1.0e20) || ( fd < 1.0e-40 && fd > 0.0))
		return false;

	return true;
}

bool
realnum_is_reasonable(float d)
{
	if ( isnan((double)d)  || !finite((double)d) )
		return false;

 	float fd = fabs(d);
	if ( (fd > 1.0e15) || ( fd < 1.0e-20 && fd > 0.0))
		return false;

	return true;
}

void
sigh()
{	cout << "Life really does suck, doesn\'t it.\n";
}

const char *
invl_version_string()
{	return "Invariant library, version 0.0";
}
