#include "invariant/stl_extend.h"
#include "invariant/assert.h"
#include <iostream.h>
#include "test.h"
#include <vector>
#include <algorithm>
#include <functional>

//	this file tests out stl_extend.h

#include "vec2.h"

class dotprod
{
	vec2 v;
    public:
	dotprod(vec2 & _v) : v(_v) {}
	double operator()(const vec2 & q) { return v*q; }
};


int
main()
{
	PR_EX( set_assert_mode(assert_msg))
	double x[] = {1.0, 2.0, 3.0, 4.0};
	double y[] = {-1.0, 0.0, 1.0, 2.0};

	PR_EXB( all_true(x, x+4,	positive<double>() )  )
	PR_EXB( all_true(x, x+4,    non_positive<double>() )  )
	PR_EXB( all_true(x, x+4,            zero<double>() )  )
	PR_EXB( all_true(x, x+4,        non_zero<double>() )  )
	PR_EXB( all_true(x, x+4,        negative<double>() )  )
	PR_EXB( all_true(x, x+4,    non_negative<double>() )  )
	cout << endl;

	PR_EXB( all_true(y, y+4,        positive<double>() )  )
	PR_EXB( all_true(y, y+4,    non_positive<double>() )  )
	PR_EXB( all_true(y, y+4,            zero<double>() )  )
	PR_EXB( all_true(y, y+4,        non_zero<double>() )  )
	PR_EXB( all_true(y, y+4,        negative<double>() )  )
	PR_EXB( all_true(y, y+4,    non_negative<double>() )  )
	cout << endl;

	PR_EXB( some_true(y, y+4,       positive<double>() )  )
	PR_EXB( some_true(y, y+4,   non_positive<double>() )  )
	PR_EXB( some_true(y, y+4,           zero<double>() )  )
	PR_EXB( some_true(y, y+4,       non_zero<double>() )  )
	PR_EXB( some_true(y, y+4,       negative<double>() )  )
	PR_EXB( some_true(y, y+4,   non_negative<double>() )  )
	cout << endl;

	PR_EXB( exactly_one_true(y, y+4,     positive<double>() )  )
	PR_EXB( exactly_one_true(y, y+4, non_positive<double>() )  )
	PR_EXB( exactly_one_true(y, y+4,         zero<double>() )  )
	PR_EXB( exactly_one_true(y, y+4,     non_zero<double>() )  )
	PR_EXB( exactly_one_true(y, y+4,     negative<double>() )  )
	PR_EXB( exactly_one_true(y, y+4, non_negative<double>() )  )
	cout << endl;

	vector<vec2> v2;
	v2.push_back(vec2(0.0,1.0));
	v2.push_back(vec2(1.0,1.0));
	v2.push_back(vec2(1.0,0.0));
	v2.push_back(vec2(1.0,-1.0));
	v2.push_back(vec2(0.0,-1.0));
	v2.push_back(vec2(-1.0,-1.0));

	vec2 t(1.0,1.0);

	// dot product of v2 and t = { 1, 2, 1, 0, -1, -2 }

	PR_EXB(all_true(v2.begin(),v2.end(),
			 positive_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(some_true(v2.begin(),v2.end(),
			 positive_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(exactly_one_true(v2.begin(),v2.end(),
			 positive_op<dotprod, vec2>(dotprod(t))  ))
	cout << endl;

	PR_EXB(all_true(v2, positive_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(some_true(v2, positive_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(exactly_one_true(v2, positive_op<dotprod, vec2>(dotprod(t))  ))
	cout << endl;

	PR_EXB(all_true(v2, zero_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(some_true(v2, zero_op<dotprod, vec2>(dotprod(t))  ))
	PR_EXB(exactly_one_true(v2, zero_op<dotprod, vec2>(dotprod(t))  ))
	cout << endl;

	PR_EXB( invl_is_sorted(x, x+4) )
	PR_EXB( invl_is_sorted(x, x+4, less<double>()) )
	PR_EXB( invl_is_sorted(x, x+4, greater<double>()) )
	PR_EXD( x[0] = 2.0; )
	PR_EXB( invl_is_sorted(x, x+4, less<double>()) )
	PR_EXB( invl_is_sorted(x, x+4, less_equal<double>()) )
	cout << endl;

	PR_EXB( invl_is_sorted(y, y+4, less<double>()) )
	PR_EXB( invl_is_sorted(y, y+4, less_equal<double>()) )
	PR_EXB( invl_is_sorted(y, y+4, greater<double>()) )
	PR_EXD( y[0] = 5.0; )
	PR_EXB( invl_is_sorted(y, y+4, less_equal<double>()) )
	PR_EX( sort(y, y+4) )
	PR_EXB( invl_is_sorted(y, y+4) )
	PR_EX( sort(y, y+4, greater<double>()) )
	PR_EXB( invl_is_sorted(y, y+4, greater<double>()) )
	PR_EXB( invl_is_sorted(y, y+4) )

	cout << "all done!" << endl;
}

